#ifndef __Rc_pwm_h__
#define __Rc_pwm_h__

#include <arduino.h>

#define LASTCHANNEL 4
#define CH_1 0
#define CH_2 1
#define CH_3 2
#define CH_4 3
#define PWM2RAD 0.002
#define RISING_EDGE 1
#define FALLING_EDGE 0
#define MINONWIDTH 950
#define MAXONWIDTH 2075
#define MINOFFWIDTH 12000
#define MAXOFFWIDTH 24000

// defines arduino pins used for receiver in arduino pin numbering schema

typedef struct {
  byte edge;
  unsigned long riseTime;
  unsigned long fallTime;
  unsigned int  lastGoodWidth;
} 
tPinTimingData;

class  Receiver {
public:
  int receiverData[LASTCHANNEL];
  int transmitterCommand[LASTCHANNEL];
  int transmitterCommandSmooth[LASTCHANNEL];
  int transmitterZero[LASTCHANNEL];
  float transmitterSmooth[LASTCHANNEL];
  Receiver(void);
  // ******************************************************************
  // The following function calls must be defined in any new subclasses
  // ******************************************************************
  virtual void initialize(void);
  virtual void read(void);
  // returns non-smoothed non-scaled ADC data in PWM full range 1000-2000 values
  const int getRaw(byte channel);
  // returns raw but smoothed receiver(channel) in PWM
  const int getRawSmoothed(byte channel);
  // returns smoothed & scaled receiver(channel) in PWM values, zero centered
  const int getData(byte channel);
  // return the smoothed & scaled number of radians/sec in stick movement - zero centered
  const float getSIData(byte channel);  
  // returns Zero value of channel in PWM
  const int getZero(byte channel);
  // sets zero value of channel in PWM
  void setZero(byte channel, int value);
  const float getSmoothFactor(byte channel);
  void setSmoothFactor(byte channel, float value);
  const float getAngle(byte channel);
  const float filterSmooth(float currentData, float previousData, float smoothFactor);
};


class Receiver_4ch : 
public Receiver {
public:
  // Configure each receiver pin for PCINT
  void initialize();

  // Calculate PWM pulse width of receiver data
  // If invalid PWM measured, use last known good time
  void read(void);
};

#endif







